import base64
import os

from Crypto.PublicKey import RSA


def generate_aes_key(size: int = 32) -> bytes:
    return base64.b64encode(os.urandom(size))


def generate_rsa_number(bits: int) -> dict:
    r = RSA.generate(bits)
    return {"p": r.p, "q": r.q, "n": r.n, "e": r.e, "d": r.d}



G = 3
M = 998244353


def bitreverse(x: list, length: int):
    j = 0
    for i in range(0, length):
        if i > j:
            x[i], x[j] = x[j], x[i]
        l = length >> 1  # noqa: E741
        while True:
            j = j ^ l
            if j >= l:
                break
            l >>= 1  # noqa: E741


def _ntt(arr: list, inverse=False):
    length = len(arr)
    if length & (length - 1) != 0:
        raise ValueError("The length of input must be a power of 2.")
    x = arr.copy()
    g = pow(G, (M - 1) // length, M)
    if inverse:
        g = pow(g, M - 2, M)
        inv = pow(length, M - 2, M)
        for i in range(length):
            x[i] = x[i] * inv % M

    bitreverse(x, length)

    w = [1] * length
    for i in range(1, length):
        w[i] = w[i - 1] * g % M
    half, step = 1, 2
    while half != length:
        wstep = length // step
        for i in range(0, length, step):
            for j in range(i, i + half):
                t = (w[wstep * (j - i)] * x[j + half]) % M
                x[j + half] = (M - t + x[j]) % M
                x[j] = (x[j] + t) % M
        half <<= 1
        step <<= 1
    return x


def ntt(arr: list):
    return _ntt(arr, False)


def intt(arr: list):
    return _ntt(arr, True)



import os
import re
import sys
from pathlib import Path
from typing import Optional

try:
    import python_minifier
except ImportError as exc:
    if sys.version_info.minor < 14:
        raise ImportError("Couldn't import python_minifier.") from exc

    python_minifier = None

from pyencrypt.aes import aes_encrypt
from pyencrypt.generate import generate_rsa_number
from pyencrypt.ntt import ntt

NOT_ALLOWED_ENCRYPT_FILES = [
    "__init__.py",
]

REMOVE_SELF_IMPORT = re.compile(r"^from pyencrypt\.[\s\S]*?$", re.MULTILINE)


def _encrypt_file(
    data: bytes,
    key: str,
) -> bytes:
    return aes_encrypt(data, key)


def can_encrypt(path: Path) -> bool:
    if path.name in NOT_ALLOWED_ENCRYPT_FILES:
        return False
    if path.suffix != ".py":
        return False
    return True


def encrypt_key(key: bytes):
    ascii_ls = [ord(x) for x in key.decode()]
    numbers = generate_rsa_number(2048)
    e, n = numbers["e"], numbers["n"]
    # fill length to be a power of 2
    length = len(ascii_ls)
    if length & (length - 1) != 0:
        length = 1 << length.bit_length()
        ascii_ls = ascii_ls + [0] * (length - len(ascii_ls))
    cipher_ls = list()
    # ntt后再用RSA加密
    for num in ntt(ascii_ls):
        cipher_ls.append(pow(num, e, n))
    return "O".join(map(str, cipher_ls)), numbers["d"], numbers["n"]


def generate_so_file(
    cipher_key: str,
    d: int,
    n: int,
    base_dir: Optional[Path] = None,
    license: bool = False,
) -> Path:
    private_key = f"{n}O{d}"
    path = Path(os.path.abspath(__file__)).parent

    decrypt_source_ls = list()
    need_import_files = ["ntt.py", "aes.py", "decrypt.py", "license.py"]
    for file in need_import_files:
        file_path = path / file
        decrypt_source_ls.append(
            REMOVE_SELF_IMPORT.sub("", file_path.read_text(encoding="utf-8"))
        )

    loader_source_path = path / "loader.py"
    loader_source = (
        REMOVE_SELF_IMPORT.sub("", loader_source_path.read_text(encoding="utf-8"))
        .replace("__private_key = None", f"__private_key = '{private_key}'", 1)
        .replace("__cipher_key = None", f"__cipher_key = '{cipher_key}'", 1)
        .replace("license = None", f"license = {license}", 1)
    )

    if base_dir is None:
        base_dir = Path(os.getcwd())

    temp_dir = base_dir / "encrypted"
    temp_dir.mkdir(exist_ok=True)
    loader_file_path = temp_dir / "loader.py"
    loader_file_path.touch(exist_ok=True)

    decrypt_source = "\n".join(decrypt_source_ls)

    # Origin file
    loader_origin_file_path = temp_dir / "loader_origin.py"
    loader_origin_file_path.touch(exist_ok=True)
    loader_origin_file_path.write_text(
        f"{decrypt_source}\n{loader_source}", encoding="utf-8"
    )

    minified_code = loader_origin_file_path.read_text(encoding="utf-8")

    if python_minifier:
        minified_code = python_minifier.minify(minified_code)

    loader_file_path.write_text(minified_code, encoding="utf-8")

    from setuptools import setup  # isort:skip
    from Cython.Build import cythonize
    from Cython.Distutils import build_ext

    setup(
        ext_modules=cythonize(loader_file_path.as_posix(), language_level="3"),
        script_args=["build_ext", "--build-lib", temp_dir.as_posix()],
        cmdclass={"build_ext": build_ext},
    )
    if sys.platform.startswith("win"):
        # loader.cp36-win_amd64.pyd
        pattern = "loader.cp*-*.pyd"
    else:
        # loader.cpython-36m-x86_64-linux-gnu.so
        # loader.cpython-36m-darwin.so
        pattern = "loader.cpython-*-*.so"

    loader_extension = next(temp_dir.glob(pattern), None)
    if loader_extension is None:
        raise Exception(f"Can't find loader extension in {temp_dir.as_posix()}")
    return loader_extension.absolute()


def encrypt_file(
    path: Path, key: str, delete_origin: bool = False, new_path: Optional[Path] = None
):
    if not can_encrypt(path):
        raise Exception(f"{path.name} can't be encrypted.")
    encrypted_data = _encrypt_file(path.read_bytes(), key)
    if new_path:
        if new_path.suffix != ".pye":
            raise Exception("Encrypted file path must be pye suffix.")
        new_path.touch(exist_ok=True)
        new_path.write_bytes(encrypted_data)
    if delete_origin:
        os.remove(path)
    return encrypted_data



import base64
import ipaddress
import os
import re
import shutil
import sys
from pathlib import Path

import click
from click.core import ParameterSource  # click>=8

from pyencrypt import __description__, __version__
from pyencrypt.decrypt import decrypt_file
from pyencrypt.encrypt import can_encrypt, encrypt_file, encrypt_key, generate_so_file
from pyencrypt.generate import generate_aes_key
from pyencrypt.license import MAX_DATETIME, MIN_DATETIME, generate_license_file

VERSION = rf"""
                                                      _
         _ __  _   _  ___ _ __   ___ _ __ _   _ _ __ | |_
        | '_ \| | | |/ _ \ '_ \ / __| '__| | | | '_ \| __|
        | |_) | |_| |  __/ | | | (__| |  | |_| | |_) | |_
        | .__/ \__, |\___|_| |_|\___|_|   \__, | .__/ \__|
        |_|    |___/                      |___/|_|

        {__description__}

                    VERSION {__version__}
"""  # noqa: E221,E222

KEY_OPTION_HELP = """
Your encryption key.If you don't specify key,
pyencrypt will generate encryption key randomly.
"""

PYTHON_MAJOR, PYTHON_MINOR = sys.version_info[:2]
LOADER_FILE_NAME = click.style("encrypted/{}", blink=True, fg="blue")
LICENSE_FILE_NAME = click.style("license.lic", blink=True, fg="blue")

SUCCESS_ANSI = click.style("successfully", fg="green")

INVALID_KEY_MSG = click.style("Your encryption 🔑 is invalid.", fg="red")

INVALID_MAC_MSG = click.style("{} is not a valid mac address.", fg="red")

INVALID_IPV4_MSG = click.style("{} is not a valid ipv4 address.", fg="red")

INVALID_DATETIME_MSG = click.style(
    "Before date must be less than after date.", fg="red"
)

FINISH_ENCRYPT_MSG = f"""
Encryption completed {SUCCESS_ANSI}.
Please copy {LOADER_FILE_NAME} into your encrypted directory.
And then remove `encrypted` directory.
Finally, add `import loader` at the top of your entry file.\n
"""  # noqa: W604

FINISH_DECRYPT_MSG = f"""
Decryption completed {SUCCESS_ANSI}. Your origin source code has be put: {{work_dir}}
"""

FINISH_GENERATE_LOADER_MSG = f"""
Generate loader file {SUCCESS_ANSI}. Your loader file is located in {LOADER_FILE_NAME}
"""

FINISH_GENERATE_LICENSE_MSG = f"""
Generate license file {SUCCESS_ANSI}. Your license file is located in {LICENSE_FILE_NAME}
"""

DATETIME_FORMATS = ["%Y-%m-%dT%H:%M:%S %z", "%Y-%m-%d %H:%M:%S", "%Y-%m-%d"]

ENVVAR_PREFIX = "PYE_ENCRYPT"


class KeyParamType(click.ParamType):
    name = "key"

    def _check_key(self, key: str) -> bool:
        return not (len(key) % 4 or len(base64.b64decode(key)) % 16)

    def convert(self, value, param, ctx) -> str:
        if ctx.get_parameter_source(param.name) == ParameterSource.ENVIRONMENT:
            visible_chars = 4
            masked = (
                value[:visible_chars]
                + "*" * (len(value) - 2 * visible_chars)
                + value[-visible_chars:]
            )
            click.echo(
                f'Using encryption key 🔑 {click.style(masked, fg="yellow")} from environment variable {click.style(param.envvar, fg="bright_cyan")}.'
            )

        value = click.STRING.convert(value, param, ctx)
        if not self._check_key(value):
            self.fail(INVALID_KEY_MSG, param, ctx)
        return value

    def get_metavar(self, param):
        return "🔑"

    def __repr__(self) -> str:
        return "KEY"


class MacAddressParamType(click.ParamType):
    name = "mac_address"
    pattern = re.compile(r"^([0-9a-fA-F]{2}[:-]){5}([0-9a-fA-F]{2})$")

    def convert(self, value, param, ctx) -> str:
        value = click.STRING.convert(value, param, ctx)
        if not self.pattern.match(value):
            self.fail(INVALID_MAC_MSG.format(value), param, ctx)
        return value

    def get_metavar(self, param):
        return "01:23:45:67:89:AB"

    def __repr__(self) -> str:
        return "MacAddress"


class IPv4AddressParamType(click.ParamType):
    name = "ipv4_address"

    def convert(self, value, param, ctx) -> str:
        value = click.STRING.convert(value, param, ctx)
        try:
            return str(ipaddress.IPv4Address(value))
        except ValueError:
            self.fail(INVALID_IPV4_MSG.format(value), param, ctx)

    def get_metavar(self, param):
        return "192.168.0.1"

    def __repr__(self) -> str:
        return "Ipv4Address"


class CustomParamType:
    KEY = KeyParamType()
    MAC_ADDR = MacAddressParamType()
    IPV4_ADDR = IPv4AddressParamType()


@click.group()
@click.version_option(__version__, "-V", "--version", message=VERSION)
@click.help_option("-h", "--help")
def cli():
    pass


@cli.command(name="encrypt")
@click.argument("pathname", type=click.Path(exists=True, resolve_path=True))
@click.option(
    "-i",
    "--in-place",
    "replace",
    default=False,
    help="make changes to files in place",
    is_flag=True,
)
@click.option(
    "-k",
    "--key",
    default=None,
    help=KEY_OPTION_HELP,
    type=CustomParamType.KEY,
    envvar=f"{ENVVAR_PREFIX}_KEY",
    show_envvar=True,
)
@click.option(
    "--with-license", default=False, help="Add license to encrypted file", is_flag=True
)
@click.option(
    "-m",
    "--bind-mac",
    "mac",
    default=None,
    help="Bind mac address to encrypted file",
    type=CustomParamType.MAC_ADDR,
)
@click.option(
    "-4",
    "--bind-ipv4",
    "ipv4",
    default=None,
    help="Bind ipv4 address to encrypted file",
    type=CustomParamType.IPV4_ADDR,
)
@click.option(
    "-b",
    "--before",
    default=MIN_DATETIME,
    help="License is invalid before this date.",
    type=click.DateTime(formats=DATETIME_FORMATS),
)
@click.option(
    "-a",
    "--after",
    default=MAX_DATETIME,
    help="License is invalid after this date.",
    type=click.DateTime(formats=DATETIME_FORMATS),
)
@click.confirmation_option(
    "-y",
    "--yes",
    prompt="Are you sure you want to encrypt your python file?",
    help="Automatically answer yes for confirm questions.",
)
@click.help_option("-h", "--help")
@click.pass_context
def encrypt_command(
    ctx, pathname, replace, key, with_license, mac, ipv4, before, after
):
    """Encrypt your python code"""
    if key is None:
        key = generate_aes_key().decode()
        click.echo(
            f'Your randomly encryption 🔑 is {click.style(key, underline=True, fg="yellow")}'
        )

    if before > after:
        ctx.fail(INVALID_DATETIME_MSG)

    path = Path(pathname)

    if path.is_file():
        if replace:
            new_path = path.with_suffix(".pye")
        else:
            new_path = Path(os.getcwd()) / path.with_suffix(".pye").name
        encrypt_file(path, key, replace, new_path)
    elif path.is_dir():
        if replace:
            work_dir = path
        else:
            work_dir = Path(os.getcwd()) / "encrypted" / path.name
            work_dir.exists() and shutil.rmtree(work_dir)
            shutil.copytree(path, work_dir)
        files = set(work_dir.glob("**/*.py"))
        with click.progressbar(files, label="🔐 Encrypting") as bar:
            for file in bar:
                new_path = file.with_suffix(".pye")
                if can_encrypt(file):
                    encrypt_file(file, key, True, new_path)
    else:
        raise Exception(f"{path} is not a valid path.")

    cipher_key, d, n = encrypt_key(key.encode())  # 需要放进导入器中
    loader_extension = generate_so_file(cipher_key, d, n, license=with_license)
    if with_license is True:
        generate_license_file(key, Path(os.getcwd()), after, before, mac, ipv4)
        click.echo(FINISH_GENERATE_LICENSE_MSG)
    click.echo(FINISH_ENCRYPT_MSG.format(loader_extension.name))


@cli.command(name="decrypt")
@click.argument("pathname", type=click.Path(exists=True, resolve_path=True))
@click.option(
    "-i",
    "--in-place",
    "replace",
    default=False,
    help="make changes to files in place",
    is_flag=True,
)
@click.option(
    "-k", "--key", required=True, help="Your encryption key.", type=CustomParamType.KEY
)
@click.help_option("-h", "--help")
@click.pass_context
def decrypt_command(ctx, pathname, replace, key):
    """Decrypt encrypted pye file"""
    path = Path(pathname)

    if path.is_file():
        if replace:
            new_path = path.with_suffix(".py")
        else:
            new_path = Path(os.getcwd()) / path.with_suffix(".py").name
        work_dir = new_path.parent
        origin_data = decrypt_file(path, key, replace, new_path)
        print(origin_data.decode())
    elif path.is_dir():
        if replace:
            work_dir = path
        else:
            work_dir = Path(os.getcwd()) / "decrypted" / path.name
            work_dir.exists() and shutil.rmtree(work_dir)
            shutil.copytree(path, work_dir)
        files = list(work_dir.glob("**/*.pye"))
        with click.progressbar(files, label="🔓 Decrypting") as bar:
            for file in bar:
                new_path = file.with_suffix(".py")
                decrypt_file(file, key, True, new_path)
    else:
        raise Exception(f"{path} is not a valid path.")

    click.echo(FINISH_DECRYPT_MSG.format(work_dir=work_dir))


@cli.command(name="generate")
@click.option(
    "-k", "--key", required=True, help="Your encryption key.", type=CustomParamType.KEY
)
@click.help_option("-h", "--help")
@click.pass_context
def generate_loader(ctx, key):
    """Generate loader file using specified key"""
    cipher_key, d, n = encrypt_key(key.encode())
    loader_extension = generate_so_file(cipher_key, d, n, Path(os.getcwd()))
    click.echo(FINISH_GENERATE_LOADER_MSG.format(loader_extension.name))


@cli.command(name="license")
@click.help_option("-h", "--help")
@click.option(
    "-k", "--key", required=True, help="Your encryption key.", type=CustomParamType.KEY
)
@click.option(
    "-m",
    "--bind-mac",
    "mac",
    default=None,
    help="Your mac address.",
    type=CustomParamType.MAC_ADDR,
)
@click.option(
    "-4",
    "--bind-ipv4",
    "ipv4",
    default=None,
    help="Your ipv4 address.",
    type=CustomParamType.IPV4_ADDR,
)
@click.option(
    "-b",
    "--before",
    default=MIN_DATETIME,
    help="License is invalid before this date.",
    type=click.DateTime(formats=DATETIME_FORMATS),
)
@click.option(
    "-a",
    "--after",
    default=MAX_DATETIME,
    help="License is invalid after this date.",
    type=click.DateTime(formats=DATETIME_FORMATS),
)
@click.pass_context
def generate_license(ctx, key, mac, ipv4, before, after):
    """Generate license file using specified key"""
    if before > after:
        ctx.fail(INVALID_DATETIME_MSG)

    generate_license_file(key, Path(os.getcwd()), after, before, mac, ipv4)
    click.echo(FINISH_GENERATE_LICENSE_MSG)


if __name__ == "__main__":
    cli()



import os
from pathlib import Path

from pyencrypt.aes import aes_decrypt
from pyencrypt.ntt import intt


def decrypt_key(cipher_key: str, d: int, n: int) -> str:
    plain_ls = list()
    for num in map(int, cipher_key.split("O")):
        plain_ls.append(pow(num, d, n))
    # 去掉intt后末尾多余的0
    return "".join(map(chr, filter(lambda x: x != 0, intt(plain_ls))))


def _decrypt_file(data: bytes, key: str) -> bytes:
    return aes_decrypt(data, key)


def decrypt_file(
    path: Path, key: str, delete_origin: bool = False, new_path: Path = None
) -> bytes:
    if path.suffix != ".pye":
        raise Exception(f"{path.name} can't be decrypted.")
    data = _decrypt_file(path.read_bytes(), key)
    if new_path:
        if new_path.suffix != ".py":
            raise Exception("Origin file path must be py suffix.")
        new_path.touch(exist_ok=True)
        new_path.write_bytes(data)
    if delete_origin:
        os.remove(path)
    return data



import hashlib
import json
import os
import socket
import subprocess
import sys
import uuid
from datetime import datetime
from pathlib import Path

from pyencrypt.aes import aes_encrypt

DATE_FORMAT = "%Y-%m-%dT%H:%M:%S%z"
MIN_DATETIME = datetime.now().astimezone()
MAX_DATETIME = datetime(year=2999, month=12, day=31).astimezone()


def get_mac_address() -> str:
    return ":".join(("%012X" % uuid.getnode())[i : i + 2] for i in range(0, 12, 2))


def get_host_ipv4() -> str:
    if sys.platform == "darwin":
        command = "ifconfig | grep 'inet ' | grep -Fv 127.0.0.1 | awk '{print $2}'"
        return subprocess.check_output(command, shell=True).decode().split()[-1]
    else:
        return socket.gethostbyname(socket.gethostname())


def get_signature(data: bytes) -> str:
    return hashlib.sha256(data).hexdigest()


FIELDS = ["invalid_before", "invalid_after", "ipv4", "mac"]


def _combine_data(data: dict) -> bytes:
    return "*".join(map(lambda x: f"{x}:{data[x]}", FIELDS)).encode()  # noqa: E231


def generate_license_file(
    aes_key: str,
    path: Path = None,
    after: datetime = None,
    before: datetime = None,
    mac_addr: str = None,
    ipv4: str = None,
    **extra,
):
    if after is None:
        after = MAX_DATETIME
    if before is None:
        before = MIN_DATETIME

    if after.tzinfo is None:
        after = after.astimezone()
    if before.tzinfo is None:
        before = before.astimezone()

    data = {
        "invalid_before": before.strftime(DATE_FORMAT),
        "invalid_after": after.strftime(DATE_FORMAT),
        "mac": mac_addr,
        "ipv4": ipv4,
    }
    encrypted_data = aes_encrypt(_combine_data(data), aes_key)
    signature = get_signature(encrypted_data)
    data.update({"signature": signature, **extra})

    if path is None:
        path = Path(os.getcwd())
    license_dir = path / "licenses"
    license_dir.mkdir(exist_ok=True)
    license_path = license_dir / "license.lic"
    license_path.touch(exist_ok=True)
    license_path.write_bytes(json.dumps(data, indent=4).encode())
    return license_path.absolute()


def check_license(license_path: Path, aes_key: str):
    if not license_path.exists():
        raise FileNotFoundError(
            f"License file {license_path.absolute().as_posix()} not found."
        )
    data = json.loads(license_path.read_text())
    signature = data.pop("signature")
    before = datetime.strptime(data["invalid_before"], DATE_FORMAT).astimezone()
    after = datetime.strptime(data["invalid_after"], DATE_FORMAT).astimezone()
    mac_address = data.get("mac")
    ipv4 = data.get("ipv4")
    now = datetime.now().astimezone()
    if signature != get_signature(aes_encrypt(_combine_data(data), aes_key)):
        raise Exception("License signature is invalid.")
    if now < before or now > after:
        raise Exception("License expired.")
    if mac_address and mac_address != get_mac_address():
        raise Exception("Machine mac address is invalid.")
    if ipv4 and ipv4 != get_host_ipv4():
        raise Exception("Machine ipv4 address is invalid.")
    return True



import linecache
import os
import sys
import traceback
import types
from functools import lru_cache
from importlib import abc, machinery
from importlib._bootstrap_external import _NamespacePath
from importlib.machinery import ModuleSpec
from importlib.util import spec_from_loader
from pathlib import Path
from typing import Iterable, Sequence, Union

from pyencrypt.decrypt import decrypt_file, decrypt_key
from pyencrypt.license import check_license

_Path = Union[bytes, str]
sys.dont_write_bytecode = True


class Base:
    def __dir__(self) -> Iterable[str]:
        return []


ENCRYPT_SUFFIX = ".pye"


class EncryptFileLoader(abc.SourceLoader, Base):
    POSSIBLE_PATH = [
        Path(os.path.expanduser("~")) / ".licenses" / "license.lic",
        Path(os.path.abspath(__file__)).parent / "licenses" / "license.lic",
        Path(os.getcwd()) / "licenses" / "license.lic",
    ]

    def __init__(self, path) -> None:
        self.path = path or ""
        self.__private_key = None
        self.__cipher_key = None
        self.license = None
        self.license_path = None
        self._init_license_path()
        self.check()

    def _init_license_path(self) -> None:
        if self.license is False:
            return
        for path in self.POSSIBLE_PATH:
            if path.exists():
                self.license_path = path
                break

    @classmethod
    @lru_cache(maxsize=128)
    def _decrypt_key(cls, cipher_key: str, d: int, n: int):
        return decrypt_key(cipher_key, d, n)

    def check(self) -> bool:
        if self.license is False:
            return False

        if self.license_path is None:
            raise Exception("Could not find license file.")

        __n, __d = self.__private_key.split("O", 1)
        check_license(
            self.license_path, self._decrypt_key(self.__cipher_key, int(__d), int(__n))
        )
        return True

    def get_filename(self, fullname: str) -> str:
        return self.path

    def get_source(self, fullname: str):
        return None

    def get_data(self, path: _Path) -> bytes:
        try:
            __n, __d = self.__private_key.split("O", 1)
            return decrypt_file(
                Path(path), self._decrypt_key(self.__cipher_key, int(__d), int(__n))
            )
        except Exception:
            traceback.print_exc()
            return b""


class EncryptFileFinder(abc.MetaPathFinder, Base):
    @staticmethod
    def _cache_line(file_path):
        stat = os.stat(file_path)
        size, mtime = stat.st_size, stat.st_mtime
        linecache.cache[file_path] = (size, mtime, [], file_path)

    @classmethod
    def find_spec(
        cls, fullname: str, path: Sequence[_Path], target: types.ModuleType = None
    ) -> ModuleSpec:
        if path:
            filename = "{}{}".format(fullname.rsplit(".", 1)[-1], ENCRYPT_SUFFIX)
            if isinstance(path, _NamespacePath):
                file_path = Path(path._path[0]) / filename
            else:
                file_path = Path(path[0]) / filename
        else:
            for p in sys.path:
                file_path = Path(p) / f"{fullname}{ENCRYPT_SUFFIX}"
                if file_path.exists():
                    break
        file_path = file_path.absolute().as_posix()
        if not os.path.exists(file_path):
            return None

        cls._cache_line(file_path)

        loader = EncryptFileLoader(file_path)
        return spec_from_loader(name=fullname, loader=loader, origin="origin-encrypt")

    @classmethod
    def invalidate_caches(cls):
        pass


# TODO: generate randomly AES Class
def _install():
    machinery.EXTENSION_SUFFIXES.append(ENCRYPT_SUFFIX)
    sys.meta_path.insert(0, EncryptFileFinder)


_install()
